//
//  PoemsGenerator.swift
//  SagePoet
//
//  Created by imac on 2020/6/27.
//  Copyright © 2020 software2018. All rights reserved.
//


import Foundation

class PoemsGernerator {

    static func  pListGenerator() -> NSDictionary? {
        guard let url = Bundle.main.url(forResource: "poems", withExtension: "txt") else {
            print("File poems.txt NOT found.")
            return nil
        }
        guard let poems = getPoemsString(from: url) else {
            print("Reading file poems.txt failure.")
            return nil
        }
        
        let poemsArray = poems.components(separatedBy: .newlines)
        
        return getPoemsPList(from: poemsArray)
        
    }
    
    
    private static func getPoemsString(from url: URL)-> String? {
        var poemsString: String?
        do {
            poemsString = try String.init(contentsOf: url, encoding: .utf8)
        } catch {
            
        }
        return poemsString
    }
    
    private static func getPoemsPList(from poemsArray: [String]) -> NSDictionary {
        var poemArray = [NSDictionary]()
        var poemCounter = 0
        
        var title: String?
        var author: String?
        var dynasty: String = ""
        
        var sentencesArray = [NSDictionary]()
        var sentenceId = 0
        
        for str in poemsArray {
            if str.count == 0 {
                continue
            }
        
            if str.contains("《") && str.contains("》") {
                poemCounter += 1
                if poemCounter >= 2 {
                    poemArray.append(getOnePoem(title: title!, dynasty: dynasty, author: author!, sentences: sentencesArray))
                }
                
                title = getPoemTitle(from: str)
                dynasty = getDynasty(from: str)
                author = getPoemAuthor(from: str)
                
                sentencesArray = []
                sentenceId = 1
            } else {
                for sentence in getPoemSentences(from: str, sentence: sentenceId) {
                    sentencesArray.append(sentence)
                }
                sentenceId += 2
            }
        }
        poemArray.append(getOnePoem(title: title!, dynasty: dynasty, author: author!, sentences: sentencesArray))
        return NSDictionary(object: poemArray, forKey: "poems" as NSCopying)
    }
    
    private static func getPoemTitle(from line: String) -> String {
        let leftIndex = line.index(after: line.firstIndex(of: "《")!)
        let rightIndex = line.index(before: line.firstIndex(of: "》")!)
        return String(line[leftIndex...rightIndex])
    }
    
    private static func getDynasty(from line: String) -> String {
        let leftIndex = line.index(after: line.firstIndex(of: "【")!)
        let rightIndex = line.index(before: line.firstIndex(of: "】")!)
        return String(line[leftIndex...rightIndex])
    }
    
    private static func getPoemAuthor(from line: String) -> String? {
        if line.contains("：") {
            let leftIndex = line.index(after: line.firstIndex(of: "：")!)
            return String(line[leftIndex..<line.endIndex])
        }
        return nil
    }
    
    private static func getPoemSentences(from line: String, sentence id: Int) -> [NSDictionary] {
        var sentences = [NSDictionary]()
        var sentenceString = ""
        var nextStart: String.Index
        if let commaIndex = line.firstIndex(of: "，") {
            sentenceString = String(line[..<commaIndex])
            nextStart = commaIndex
        } else if let questionIndex = line.firstIndex(of: "？") {
            sentenceString = String(line[..<questionIndex])
            nextStart = questionIndex
        } else {
            nextStart = line.firstIndex(of: " ")!
        }
        var sentenceKey = String.localizedStringWithFormat("s%i", id)
        var sentence = NSDictionary(object: sentenceString, forKey: sentenceKey as NSCopying)
        sentences.append(sentence)
        
        if let periodIndex = line.firstIndex(of: "。") {
            sentenceString = String(line[line.index(after: nextStart)..<periodIndex])
        } else if let questionIndex = line.firstIndex(of: "？") {
            sentenceString = String(line[line.index(after: nextStart)..<questionIndex])
        } else if let exclamatoryIndex = line.firstIndex(of: "！") {
            sentenceString = String(line[line.index(after: nextStart)..<exclamatoryIndex])
        }
        
        sentenceKey = String.localizedStringWithFormat("s%i", id + 1)
        sentence = NSDictionary(object: sentenceString, forKey: sentenceKey as NSCopying)
        sentences.append(sentence)
        return sentences
    }
    
    private static func getOnePoem(title: String, dynasty: String, author: String, sentences: [NSDictionary]) -> NSDictionary {
        let poem = NSMutableDictionary()
        poem.setValue(title, forKey: "title")
        poem.setValue(author, forKey: "author")
        poem.setValue(dynasty, forKey: "dynasty")
        poem.setValue(sentences.count, forKey: "totalLines")
        poem.setValue(sentences, forKey: "sentences")
        return poem
    }
}


